using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace AIOWDGNet
{
	public class AIOWDG
	{
		public const WDG_ACTION_IGNORE = 0
		public const WDG_ACTION_DISABLE = 1
		public const WDG_ACTION_SOFT_RESTART = 2
		public const WDG_ACTION_MOSTLY_SOFT_RESTART = 4

		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 GetNumCards();
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 QueryCardInfo(UInt32 CardNum, ref UInt32 pDeviceID, ref UInt32 pBase, ref UInt32 pNameSize, IntPtr pName);

		public static UInt32 QueryCardInfo(UInt32 CardNum, ref UInt32 pDeviceID, ref UInt32 pBase, ref UInt32 pNameSize, ref string Name)
		{
			UInt32 Result;
			IntPtr pName = Marshal.AllocHGlobal(pNameSize);

			Result = QueryCardInfo(CardNum, ref pDeviceID, ref pBase, ref pNameSize, pName);
			Name = Marshal.PtrToStringAnsi(pName);
			Name = Name.Substring(0, Math.Min(Name.Length,pNameSize));

			return Result;
		}

		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 QueryCardID(UInt32 CardNum, ref UInt32 pDeviceID);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 QueryCardName(UInt32 CardNum, ref UInt32 pNameSize, IntPtr pName);

		public static UInt32 QueryCardName(UInt32 CardNum, ref UInt32 pNameSize, ref string pName)
		{
			UInt32 Result;
			IntPtr pName = Marshal.AllocHGlobal(pNameSize);

			Result = QueryCardName(CardNum, ref pNameSize, pName);
			Name = Marshal.PtrToStringAnsi(pName);
			Name = Name.Substring(0, Math.Min(Name.Length,pNameSize));

			return Result;
		}

		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 QuerySerialNumber(UInt32 CardNum, ref UInt64 pSerialNumber);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 AIOWDG_CloseDevice(UInt32 CardNum);

		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 GetDeviceBySerialNumber(ref UInt64 pSerialNumber);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 GetDeviceByEEPROMByte(Byte Data);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 GetDeviceByEEPROMData(UInt32 StartAddress, UInt32 DataSize, ref Byte pData);

		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 CustomEEPROMRead(UInt32 CardNum, UInt32 StartAddress, ref UInt32 DataSize, [Out] Byte[] Data);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 CustomEEPROMWrite(UInt32 CardNum, UInt32 StartAddress, UInt32 DataSize, [In] Byte[] Data);

		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 WaitForIRQ(UInt32 CardNum);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 AbortRequest(UInt32 CardNum);

		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 WDGInit(UInt32 CardNum);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 WDGSetTimeout(UInt32 CardNum, ref double pMilliseconds);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 WDGSetResetDuration(UInt32 CardNum, ref double pMilliseconds);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 WDGPet(UInt32 CardNum);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 WDGReadTemp(UInt32 CardNum, ref double TempF);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 WDGReadStatus(UInt32 CardNum, ref UInt32 StatusByte);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 WDGStart(UInt32 CardNum);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 WDGStop(UInt32 CardNum);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 WDGHandleIRQ(UInt32 CardNum, UInt32 Action);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 EmergencyReboot();

		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 DI_ReadAll(UInt32 CardNum, ref Byte pDIData);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 DI_Read1(UInt32 CardNum, Byte BitIndex, ref UInt32 pBitValue);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 DI_ReadLatched(UInt32 CardNum, ref Byte pDIRisen, ref Byte pDIFallen);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 DI_PeekAll(UInt32 CardNum, ref Byte pDIData);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 DI_Peek1(UInt32 CardNum, Byte BitIndex, ref UInt32 pBitValue);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 DI_SetDebounceAll(UInt32 CardNum, Byte TimeMS);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 DI_SetDebounce1(UInt32 CardNum, Byte BitIndex, Byte TimeMS);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 DI_SetDebounceSome(UInt32 CardNum, Byte BitMask, Byte TimeMS);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 DO_WriteAll(UInt32 CardNum, Byte DOData);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 DO_Write1(UInt32 CardNum, Byte BitIndex, UInt32 BitValue);

		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 AI_ReadAll(UInt32 CardNum, ref double pAIData);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 AI_Read1(UInt32 CardNum, Byte ChannelIndex, ref double pAIData);



		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 GenericVendorRead(UInt32 CardNum, Byte Request, UInt16 Value, UInt16 Index, ref UInt32 DataSize, [Out] Byte[] pData);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 GenericVendorWrite(UInt32 CardNum, Byte Request, UInt16 Value, UInt16 Index, UInt32 DataSize, [In] Byte[] pData);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 AWU_GenericBulkIn(UInt32 CardNum, UInt32 PipeID, [Out] Byte[] pData, UInt32 DataSize, ref UInt32 BytesRead);
		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 AWU_GenericBulkOut(UInt32 CardNum, UInt32 PipeID, [In] Byte[] pData, UInt32 DataSize, ref UInt32 BytesWritten);

		[DllImport("AIOWDG.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
		public static extern UInt32 AIOWDG_ResetChip(UInt32 CardNum);

		//Note that all error codes returned are Windows error codes; they can be found in winerror.h in the Windows SDK, or on the Web.
	}
}

